@@ -6,7 +6,7 @@
 
 
 void ngx_http_lua_inject_resp_header_api(lua_State *L);
-void ngx_http_lua_inject_req_header_api(lua_State *L);
+void ngx_http_lua_inject_req_header_api(ngx_log_t *log, lua_State *L);
 
 
 #endif /* NGX_HTTP_LUA_HEADERS_H */
