@@ -1,9 +1,9 @@
-#!/bin/bash
+#!/usr/bin/env bash
 
 # this file is mostly meant to be used by the author himself.
 
 root=`pwd`
-version=${1:-1.0.5}
+version=${1:-1.2.1}
 home=~
 force=$2
 
@@ -13,14 +13,17 @@ force=$2
 
             #--without-pcre \
             #--without-http_rewrite_module \
+            #--without-http_autoindex_module \
             #--with-cc=gcc46 \
+            #--with-cc=clang \
 
 time ngx-build $force $version \
+            --with-cc-opt="-I$PCRE_INC" \
+            --with-http_realip_module \
         --with-http_ssl_module \
             --add-module=$root/../ndk-nginx-module \
             --add-module=$root/../set-misc-nginx-module \
-            --with-cc-opt=$'-I/opt/pcre821jit/include' \
-            --with-ld-opt="-L/opt/pcre821jit/lib -Wl,-rpath,/opt/pcre821jit/lib:/opt/drizzle/lib:/home/lz/lib:/usr/local/openresty/luajit/lib:/usr/local/lib" \
+            --with-ld-opt="-L$PCRE_LIB -Wl,-rpath,$PCRE_LIB:$LIBDRIZZLE_LIB:$LUAJIT_LIB:/usr/local/lib" \
             --without-mail_pop3_module \
             --without-mail_imap_module \
             --without-mail_smtp_module \
@@ -28,17 +31,19 @@ time ngx-build $force $version \
             --without-http_empty_gif_module \
             --without-http_memcached_module \
             --without-http_referer_module \
-            --without-http_autoindex_module \
             --without-http_auth_basic_module \
             --without-http_userid_module \
                 --add-module=$home/work/nginx/ngx_http_auth_request_module-0.2 \
                 --add-module=$root/../echo-nginx-module \
                 --add-module=$root/../memc-nginx-module \
+                --add-module=$root/../srcache-nginx-module \
                 --add-module=$root \
               --add-module=$root/../headers-more-nginx-module \
                 --add-module=$root/../drizzle-nginx-module \
-                --add-module=$home/work/nginx/ngx_http_upstream_keepalive-2ce9d8a1ca93 \
+                --add-module=$home/work/nginx/ngx_http_upstream_keepalive-0.7 \
                 --add-module=$root/../rds-json-nginx-module \
+                --add-module=$root/../coolkit-nginx-module \
+                --add-module=$root/../redis2-nginx-module \
           --with-select_module \
           --with-poll_module \
           --with-rtsig_module \
